@@ -46,7 +46,7 @@ person.
 =item * id
 
 This method should return a unique identifier for the current person.
-The identifier should be a number.
+The identifier can be a string or a number.
 
 =item * gender
 
@@ -73,6 +73,20 @@ be significant.
 I realise that this is a significant limitation and I'll be thinking about
 how to fix it as soon as possible.
 
+=back
+
+=head2 Constructor
+
+The constructor for this class takes one optional attribute called `abbr`.
+The default value for this attribute is 2. When set, strings of repeated
+"great"s in a relationship description will collapsed to "$n x great".
+
+For example, if the description you have is "Great, great, great
+grandfather", then that will be abbreviated to to "3 x great grandfather".
+
+The value for `abbr` is the maximum number of repetitions that will be left
+untouched. You can turn abbreviations off by setting `abbr` to zero.
+
 =head2 Caching
 
 Calculating relationship names isn't at all different. But there can be a lot
@@ -87,12 +101,12 @@ consider putting a caching layer in front of C<get_relationship>.
 package Genealogy::Relationship;
 
 use Moo;
-use Types::Standard qw[Str HashRef];
+use Types::Standard qw[Int Str HashRef];
 use List::Util qw[first];
 use List::MoreUtils qw[firstidx];
 use Lingua::EN::Numbers qw[num2en num2en_ordinal];
 
-our $VERSION = '0.1.2';
+our $VERSION = '0.2.0';
 
 has parent_field_name => (
   is => 'ro',
@@ -139,6 +153,12 @@ sub _build_relationship_table {
   };
 }
 
+has abbr => (
+  is => 'ro',
+  isa => Int,
+  default => 3,
+);
+
 =head1 Methods
 
 The following methods are defined.
@@ -155,14 +175,14 @@ sub most_recent_common_ancestor {
   my ($person1, $person2) = @_;
 
   # Are they the same person?
-  return $person1 if $person1->id == $person2->id;
+  return $person1 if $person1->id eq $person2->id;
 
   my @ancestors1 = ($person1, $self->get_ancestors($person1));
   my @ancestors2 = ($person2, $self->get_ancestors($person2));
 
   for my $anc1 (@ancestors1) {
     for my $anc2 (@ancestors2) {
-      return $anc1 if $anc1->id == $anc2->id;
+      return $anc1 if $anc1->id eq $anc2->id;
     }
   }
 
@@ -205,12 +225,40 @@ sub get_relationship {
 
   my ($x, $y) = $self->get_relationship_coords($person1, $person2);
 
+  my $rel;
+
   if (defined $self->relationship_table->{$person1->gender}[$x][$y]) {
-    return $self->relationship_table->{$person1->gender}[$x][$y];
+    $rel = $self->relationship_table->{$person1->gender}[$x][$y];
   } else {
-    return $self->relationship_table->{$person1->gender}[$x][$y] =
+    $rel = $self->relationship_table->{$person1->gender}[$x][$y] =
       ucfirst $self->make_rel($person1->gender, $x, $y);
   }
+
+  $rel = $self->abbr_rel($rel) if $self->abbr;
+
+  return $rel;
+}
+
+=head2 abbr_rel
+
+Optionally abbreviate a relationship description.
+
+=cut
+
+sub abbr_rel {
+  my $self = shift;
+  my ($rel) = @_;
+
+  return $rel unless $self->abbr;
+
+  my @greats = $rel =~ /(great)/gi;
+  my $count  = @greats;
+
+  return $rel if $count < $self->abbr;
+
+  $rel =~ s/(great,\s+)+/$count x /i;
+
+  return $rel;
 }
 
 =head2 make_rel
@@ -302,20 +350,54 @@ sub get_relationship_coords {
   my ($person1, $person2) = @_;
 
   # If the two people are the same person, then return (0, 0).
-  return (0, 0) if $person1->id == $person2->id;
+  return (0, 0) if $person1->id eq $person2->id;
 
   my @ancestors1 = ($person1, $self->get_ancestors($person1));
   my @ancestors2 = ($person2, $self->get_ancestors($person2));
 
   for my $i (0 .. $#ancestors1) {
     for my $j (0 .. $#ancestors2) {
-      return ($i, $j) if $ancestors1[$i]->id == $ancestors2[$j]->id;
+      return ($i, $j) if $ancestors1[$i]->id eq $ancestors2[$j]->id;
     }
   }
 
   die "Can't work out the relationship.\n";
 }
 
+=head2 get_relationship_ancestors
+
+Given two people, returns lists of people linking those two people
+to their most recent common ancestor.
+
+The return value is a reference to an array containing two array
+references. The first references array contains the person1 and
+all their ancestors up to an including the most recent common
+ancestor. The second list does the same for person2.
+
+=cut
+
+sub get_relationship_ancestors {
+  my $self = shift;
+  my ($person1, $person2) = @_;
+
+  my $mrca = $self->most_recent_common_ancestor($person1, $person2)
+    or die "There is no most recent common ancestor\n";
+
+  my (@ancestors1, @ancestors2);
+
+  for ($person1, $self->get_ancestors($person1)) {
+    push @ancestors1, $_;
+    last if $_->id eq $mrca->id;
+  }
+
+  for ($person2, $self->get_ancestors($person2)) {
+    push @ancestors2, $_;
+    last if $_->id eq $mrca->id;
+  }
+
+  return [ \@ancestors1, \@ancestors2 ];
+}
+
 =head1 AUTHOR
 
 Dave Cross <dave@perlhacks.com>
